// Copyright (C) 1996 DIMACS Center, Rutgers, The State University of New Jersey
// Author(s): Jonathan Berry

// This software is copyrighted by the DIMACS Center at Rutgers, The State
// University of New Jersey.  IT IS PROVIDED AS IS, AND THE AUTHORS, DIMACS, AND
// RUTGERS, THE STATE UNIVERSITY OF NEW JERSEY  DISCLAIM
// ALL LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
// DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
// IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
// NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
// MODIFICATIONS.

// The authors hereby grant permission to use, copy, modify, distribute,
// and license this software and its documentation for any purpose, provided
// that existing copyright notices are retained in all copies and that this
// notice is included verbatim in any distributions. No written agreement,
// license, or royalty fee is required for any of the authorized uses.
// Modifications to this software may be copyrighted by their authors
// and need not follow the licensing terms described here, provided that
// the new terms are clearly indicated on the first page of each file where
// they apply.

// Last File Update: 31-Jul-1996
// 

#ifndef ALGORITHMS_H
#define ALGORITHMS_H

#include <LINK/basic/Set.h>
#include <LINK/basic/Sequence.h>
#include <LINK/basic/Matrix.h>
#include <LINK/graph/Graph.h>
#include <LINK/graph/Vertex.h>
#include <LINK/graph/Edge.h>

#ifdef STK_GUI		// just constructing libalg.a - won't link with STk
#include <stk.h>
char * getViewName(SCM);
#endif

#define ALG_STEP	0
#define INIT_STEP	1
#define LIFO            0
#define GLOBAL          1
#define FIFO            2
#define MAXIMUM_EXCESS  3
#define GAP             4


typedef Vertex* VertexPtr;
typedef int (*VertexComparison)(const VertexPtr&, const VertexPtr&);

void starDrawGraph(char *);
void layoutVertices(char *);
void layoutEdges(char *);

int initAnimation();
void animationWindows(int k, char *view_name);
void animationDone(int id);
void animationRun(int id);
void animationContinue(int id);
void animationStep(int id);
void animationBackup(int id);
void animationClearBreaks(int id);
void animationBreak(int id);

template <class Item>
void animateVertex(Vertex*, char *, char *, char *, int, const Item&);
template <class Item>
void animateEdge(Edge*, char *, char *, char *, int, const Item&);

void 			ResetAttributes(Graph*);
Set< Set<Vertex*> >	BiconnectedComponents(Graph*, char*graph_view=0);
Sequence<Vertex*> 	BreadthFirstSearch(Graph*, char*graph_view=0,
					   Vertex *v=0);
Sequence<Vertex*> 	DepthFirstSearch(Graph*, char*graph_view=0, 
				   VertexComparison compare=0);
void 			FloydWarshall(Graph*, AsymMatrix<double>&);
AsymMatrix<double> 	FloydWarshall(Graph*, char*graph_view=0);
int 			GoldbergTarjan(Graph*, int, int, int);
void 			HamiltonianCycles(Graph*, char*graph_view=0);
Set<Edge*> 		Kruskal(Graph*, char*graph_view=0);
void 			LatkaCycles(Graph*, char*graph_view=0);
Set<Vertex*> 		MaximalClique(Graph*, char*graph_view=0);
Set< Set<Vertex*> > 	StronglyConnectedComponents(Graph*, char* graph_view=0);
Sequence<Vertex*> 	TopologicalSort(Graph*, char* graph_view=0);
void 			Triangles(Graph*, char*graph_view=0);

#endif
