// Copyright (C) 1996 DIMACS Center, Rutgers, The State University of New Jersey
// Author(s): Jonathan Berry

// This software is copyrighted by the DIMACS Center at Rutgers, The State
// University of New Jersey.  IT IS PROVIDED AS IS, AND THE AUTHORS, DIMACS, AND
// RUTGERS, THE STATE UNIVERSITY OF NEW JERSEY  DISCLAIM
// ALL LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
// DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
// IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
// NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
// MODIFICATIONS.

// The authors hereby grant permission to use, copy, modify, distribute,
// and license this software and its documentation for any purpose, provided
// that existing copyright notices are retained in all copies and that this
// notice is included verbatim in any distributions. No written agreement,
// license, or royalty fee is required for any of the authorized uses.
// Modifications to this software may be copyrighted by their authors
// and need not follow the licensing terms described here, provided that
// the new terms are clearly indicated on the first page of each file where
// they apply.

// Last File Update: 31-Jul-1996
// 

#ifndef GraphObject_h
#define GraphObject_h

#include <fstream.h>
#include <LINK/basic/List.h>
#include <LINK/basic/stdtypes.h>
#include <LINK/basic/general.h>

class AttributeBase;
class Graph;
class Graphic;
class GraphView;

class GraphObject {
protected:
    GraphObject(Graph* passed_owner);
    GraphObject(const GraphObject &G, Flag clone=0);
    GraphObject() : _owner((Graph*) this) { assert(FALSE); } 
				// Assumes that the object is a graph
				// but this needs to be checked!

public:
    virtual ~GraphObject();	// should be protected, but needs to be
				// public for the automation of attribute
				// value deletion (see genOverloadAtt..
				// in autowrap/instantiate.cc)
    virtual DataType	type() const = 0;

    void		addAttr(AttributeBase* passed_attribute);
    void		removeAttr(AttributeBase* passed_attribute);
    void		deleteAttr(String passed_name);

    AttributeBase*	getAttr(String passed_name) const;
    AttributeBase*	findAttr(String passed_name);

    void		printAttr(String passed_name);
    void		printAttr();

    String		name() const;
    Graph*		owner()	const			{ return _owner; }
    const List<AttributeBase*>& attributes() const	{ return _attributes; }

    void                saveToFile(ofstream* fout, int indent);

    Bool                operator<(const GraphObject& obj);
    Bool                operator>(const GraphObject& obj);
    Bool                operator==(const GraphObject& obj);
    Bool                operator!=(const GraphObject& obj);
    virtual ostream& 	display(ostream& os) const = 0;
    friend ostream& operator<<(ostream& os, const GraphObject& g);


protected:
    Graph*		_owner;
    List<AttributeBase*> _attributes;
    static Graph*	_graphBeingConstructed;
};


#endif
