// Copyright (C) 1996 DIMACS Center, Rutgers, The State University of New Jersey
// Author(s): Elizabeth Johnson (Indiana University) Jonathan Berry

// This software is copyrighted by the DIMACS Center at Rutgers, The State
// University of New Jersey.  IT IS PROVIDED AS IS, AND THE AUTHORS, DIMACS, AND
// RUTGERS, THE STATE UNIVERSITY OF NEW JERSEY  DISCLAIM
// ALL LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
// DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
// IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
// NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
// MODIFICATIONS.

// The authors hereby grant permission to use, copy, modify, distribute,
// and license this software and its documentation for any purpose, provided
// that existing copyright notices are retained in all copies and that this
// notice is included verbatim in any distributions. No written agreement,
// license, or royalty fee is required for any of the authorized uses.
// Modifications to this software may be copyrighted by their authors
// and need not follow the licensing terms described here, provided that
// the new terms are clearly indicated on the first page of each file where
// they apply.

// Last File Update: 31-Jul-1996
// 

#include <fstream.h>
#include <LINK/graph/Graph.h>
#include <LINK/graph/Vertex.h>
#include <LINK/graph/Edge.h>
#include <LINK/graph/Attribute.h>
#include <stdlib.h>
    

int
SaveDimacsGraph(Graph* graph,char* filename)
{
  ofstream outFile;

  outFile.open(filename,ios::out);

  if(outFile.fail()) {
    cout << "Error: could not open file" << endl;
    return 1;
  }

  outFile << "c DIMACS GRAPH" << endl;

  Edge *edge;
  const MSet<Edge*>& edges = graph->edges();
  Iterator<Edge*> get_edge(&edges);
  Vertex** vertices=graph->vertexStart();
  Attribute<int> index_attr(graph, "index", 0);

  int n=graph->order();
  int m=graph->size();

  int i;
  for(i=0; i < n; i++) 
    setAttribute((GraphObject*) vertices[i], "index", i);    

  outFile << "p edge " << n << " " << m << endl; 

  while (get_edge(edge)) {
    Vertex* source=edge->sourceVertex();
    int source_index;
    getAttribute((GraphObject*) source,"index",source_index);
    Vertex* sink=edge->sinkVertex();
    int sink_index;
    getAttribute((GraphObject*) sink,"index",sink_index);
    // currently only writes out "weight" attribute
    // should write out choice
// now even that it commented out due to problems with attributes
//    int weight;
 //   getAttribute((GraphObject*) edge,"weight",weight);
    outFile << "e " << source_index+1 << " " << sink_index+1 << " " 
//      << weight << endl;
     << 0 << endl;
  }
  outFile.close();
  return 0;
}
